/* Zodiac - Font.cpp
 */

#include "Font.h"
#include <assert.h>
#include <stdarg.h>

// The next two functions are borrowed from the gl_font.c test program from SDL_ttf
static int powerOfTwo(int input)
{
	int value = 1;

	while (value < input)
	{
		value <<= 1;
	}
	return value;
}

int copySurfaceToTexture(SDL_Surface *surface, GLuint texture, GLfloat *texCoords)
{
	int w, h;
	SDL_Surface *image;
	SDL_Rect area;
	Uint32 saved_flags;
	Uint8 saved_alpha;

	// Use the surface width & height expanded to the next powers of two
	w = powerOfTwo(surface->w);
	h = powerOfTwo(surface->h);
	texCoords[0] = 0.0f;
	texCoords[1] = 0.0f;
	texCoords[2] = (GLfloat)surface->w / w;
	texCoords[3] = (GLfloat)surface->h / h;

#if SDL_BYTEORDER == SDL_LIL_ENDIAN
	image = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32, 0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000);
#else
	image = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32, 0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff);
#endif

	if(image == NULL)
	{
		return 0;
	}

	// Save the alpha blending attributes
	saved_flags = surface->flags&(SDL_SRCALPHA|SDL_RLEACCELOK);
	saved_alpha = surface->format->alpha;
	if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA )
	{
		SDL_SetAlpha(surface, 0, 0);
	}

	// Copy the surface into the texture image
	area.x = 0;
	area.y = 0;
	area.w = surface->w;
	area.h = surface->h;
	SDL_BlitSurface(surface, &area, image, &area);

	// Restore the blending attributes
	if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA )
	{
		SDL_SetAlpha(surface, saved_flags, saved_alpha);
	}

	// Send the texture to OpenGL
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D,	0, GL_RGBA,	w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE,	image->pixels);

	// Don't need the extra image anymore
	SDL_FreeSurface(image);

	return 1;
}

Font::Font(TTF_Font *f)
{
	assert(f != NULL && "Font cannot be created with a null TTF_Font!");
	m_ttfFont = f;
	m_texture = 0;
}

Font::~Font()
{
	if (m_ttfFont != NULL)
	{
		TTF_CloseFont(m_ttfFont);
		m_ttfFont = NULL;
	}

	if (m_texture != 0)
	{
		glDeleteTextures(1, &m_texture);
	}
}

Font* Font::loadFromFile(const char *fileName, int pointSize)
{
	// Make sure the TTF library was initialized
	if(!TTF_WasInit())
	{
		if(TTF_Init() != -1)
		{
			atexit(TTF_Quit);
		} else
		{
			printf("Font::loadFontFromFile: Could not initialize SDL_TTF!\n");
			return false;
		}
	}

	// Try and open the font
	TTF_Font *sdlFont = TTF_OpenFont(fileName, pointSize);
	if(!sdlFont)
	{
		// couldn't open it, for one reason or another
		return NULL;
	}

	Font *font = new Font(sdlFont);
	return font;
}

void Font::drawText(int x, int y, const char *text)
{
	SDL_Surface *textSurf;
	SDL_Color color = { 0xFF, 0xFF, 0xFF, 0 };

	if (m_ttfFont == NULL) return;

	// Let TTF render the text
	textSurf = TTF_RenderText_Blended(m_ttfFont, text, color);

	// Does this font already have a texture?  If not, allocate it here
	if(m_texture == 0)
	{
		glGenTextures(1, &m_texture);
	}

	// Copy the surface to the texture
	if(copySurfaceToTexture(textSurf, m_texture, m_texCoords))
	{
		// And draw the darn thing
		glBegin(GL_TRIANGLE_STRIP);
			glTexCoord2f(m_texCoords[0], m_texCoords[1]);
			glVertex2i(x, y);
			glTexCoord2f(m_texCoords[2], m_texCoords[1]);
			glVertex2i(x + textSurf->w, y);
			glTexCoord2f(m_texCoords[0], m_texCoords[3]);
			glVertex2i(x, y + textSurf->h);
			glTexCoord2f(m_texCoords[2], m_texCoords[3]);
			glVertex2i(x + textSurf->w, y + textSurf->h);
		glEnd();
	}

	// Done with the surface
	SDL_FreeSurface(textSurf);
}

void Font::drawTextFormatted(int x, int y, const char *format, ...)
{
	va_list args;
	char buffer[256];

	va_start(args, format);
	_vsnprintf(buffer, 256, format, args);
	va_end(args);

	drawText(x, y, buffer);
}

void Font::getTextSize(const char *text, int &width, int &height)
{
	if (m_ttfFont)
	{
		TTF_SizeText(m_ttfFont, text, &width, &height);
	}
}
