/* Zodiac - UiContext.cpp
 */

#include "UiContext.h"
#include <SDL.h>
#include <SDL_opengl.h>

UiContext::UiContext()
{
	m_activeControl = UI_Nothing;
	m_hotControl = UI_Nothing;
	m_font = NULL;
	m_mouseBeingDragged = false;
	m_mousePressed = m_mouseReleased = false;
}

UiContext::~UiContext()
{
	delete m_font;
}

void UiContext::setFont(Font *font)
{
	m_font = font;
}

void UiContext::setActiveControl(UI_ID control)
{
	m_activeControl = control;
}

void UiContext::setHotControl(UI_ID control)
{
	m_hotControl = control;
}

UI_ID UiContext::activeControl() const
{
	return m_activeControl;
}

UI_ID UiContext::hotControl() const
{
	return m_hotControl;
}

bool UiContext::mouseInsideRect(UiRect rect) const
{
	int right, bottom;
	right = rect.width + rect.x;
	bottom = rect.height + rect.y;

	if (m_mousePosition.x >= rect.x &&
		m_mousePosition.y >= rect.y &&
		m_mousePosition.x <= right &&
		m_mousePosition.y <= bottom)
	{
		return true;
	}
	return false;
}

void UiContext::handleSDLEvent(SDL_Event *evt)
{
	if (!evt) return;

	switch(evt->type)
	{
	case SDL_MOUSEBUTTONDOWN:
		m_mouseReleased = false;
		m_mousePressed = true;
		break;

	case SDL_MOUSEBUTTONUP:
		m_mousePressed = false;
		m_mouseReleased = true;
		break;

	case SDL_MOUSEMOTION:
		m_mousePosition.x = evt->motion.x;
		m_mousePosition.y = evt->motion.y;
		break;
	}
}

void UiContext::beginFrame(float deltaTime)
{
	// Setup the screen for drawing in 2D
	SDL_Surface *displaySurface = SDL_GetVideoSurface();
	if (!displaySurface) return;

	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_CULL_FACE);
	glEnable(GL_TEXTURE_2D);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glViewport(0, 0, displaySurface->w, displaySurface->h);

	// Save the current modelview matrix, and start with a fresh one
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	// Set up an orthographic projection for the gui to use.  Controls are free
	// to modify this, but most of them will be working in flatland, so it's
	// best to default to that
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0, displaySurface->w, displaySurface->h, 0, -1, 1);

	// Hotness gets reset at the start of each frame
	m_hotControl = UI_Nothing;
}

void UiContext::endFrame()
{
	// Reset the screen to what it was before beginFrame was called
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	// Re-enable any states disabled by beginFrame
	glPopAttrib();

	// Determine whether the mouse was dragged in this frame.  Do not start dragging
	// if a button was clicked on, however.
	if (m_mousePressed && m_activeControl == UI_Nothing)
	{
		m_mouseBeingDragged = true;
	} else if(m_mouseReleased)
	{
		m_mouseBeingDragged = false;
	}

	// Clear input states at the end of the frame
	m_mouseLastPosition = m_mousePosition;
	m_mousePressed = m_mouseReleased = false;
}

UiPoint UiContext::mouseDragDistance() const
{
	if (m_mouseBeingDragged)
	{
		return UiPoint(
			m_mousePosition.x - m_mouseLastPosition.x,
			m_mousePosition.y - m_mouseLastPosition.y);
	} else
	{
		return UiPoint(0, 0);
	}
}

bool UiContext::doButton(UI_ID id, const char *label, UiRect rect)
{
	bool result = buttonLogic(id, rect);

	bool active = activeControl() == id;
	bool hot = hotControl() == id;

	drawButton(rect, active, hot);

	// And then draw the text that goes on top of the button
	if(m_font)
	{
		int text_x, text_y;
		int text_w, text_h;

		// find the width & height of the text to be drawn, so that it can be centered inside
		// the button
		m_font->getTextSize(label, text_w, text_h);

		text_x = (rect.width - text_w) / 2 + rect.x;
		text_y = (rect.height - text_h) / 2 + rect.y;

		glColor3f(1, 1, 1);
		m_font->drawText(text_x, text_y, label);
	}

	return result;
}

bool UiContext::buttonLogic(UI_ID id, UiRect rect)
{
	bool result = false;

	// If the mosue is over the button, try and set hotness so that it can be clicked
	if (mouseInsideRect(rect))
	{
		setHotControl(id);
	}

	// Check to see if the button gets clicked on
	if (activeControl() == id)
	{
		if (mouseReleased())
		{
			if (hotControl() == id) result = true;
			setActiveControl(UI_Nothing);
		}
	} else if (hotControl() == id)
	{
		if (mousePressed())
		{
			setActiveControl(id);
		}
	}

	return result;
}

void UiContext::drawButton(UiRect rect, bool active, bool hot) const
{
	// Draw the button
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);
	{
		if(active && hot)
			glColor4f(0, 0, 0.9f, 0.6f);
		else if(hot)
			glColor4f(0.9f, 0, 0, 0.6f);
		else
			glColor4f(0.6f, 0, 0, 0.6f);

		glVertex2i(rect.x, rect.y);
		glVertex2i(rect.x, rect.y + rect.height);
		glVertex2i(rect.x + rect.width, rect.y + rect.height);
		glVertex2i(rect.x + rect.width, rect.y);
	}
	glEnd();

	glEnable(GL_TEXTURE_2D);
}

void UiContext::drawImage(Texture *img, UiRect rect) const
{
	int w, h;
	float x1, y1;

	if(img)
	{
		w = rect.width ? rect.width : img->sourceWidth();
		h = rect.height ? rect.height : img->sourceHeight();

		x1 = (float)img->sourceWidth() / img->width();
		y1 = (float)img->sourceHeight() / img->height();

		// Draw the image
		glBindTexture(GL_TEXTURE_2D, img->binding());
		glBegin(GL_TRIANGLE_STRIP);
			glTexCoord2f(0, 0);		glVertex2i(rect.x, rect.y);
			glTexCoord2f(x1, 0);	glVertex2i(rect.x + w, rect.y);
			glTexCoord2f(0, y1);	glVertex2i(rect.x, rect.y + h);
			glTexCoord2f(x1, y1);	glVertex2i(rect.x + w, rect.y + h);
		glEnd();
	}
}
